@@ -38,13 +38,13 @@
 #include <stdio.h>
 #include <OpenIPMI/ipmiif.h>
 #include <OpenIPMI/ipmi_mc.h>
+#include <OpenIPMI/ipmi_err.h>
 #include <OpenIPMI/ipmi_cmdlang.h>
 #include <OpenIPMI/ipmi_fru.h>
 #include <OpenIPMI/ipmi_conn.h>
 
 /* Internal includes, do not use in your programs */
 #include <OpenIPMI/internal/ipmi_malloc.h>
-#include <OpenIPMI/internal/ipmi_event.h>
 
 /* Don't pollute the namespace iwth ipmi_fru_t. */
 void ipmi_cmdlang_dump_fru_info(ipmi_cmd_info_t *cmd_info, ipmi_fru_t *fru);
@@ -86,7 +86,7 @@ domain_info(ipmi_domain_t *domain, void *cb_data)
     ipmi_cmdlang_down(cmd_info);
     ipmi_cmdlang_out(cmd_info, "Name", domain_name);
     if (ipmi_domain_get_guid(domain, guid) == 0)
-	ipmi_cmdlang_out_binary(cmd_info, "GUID", guid, 16);
+	ipmi_cmdlang_out_binary(cmd_info, "GUID", (char *) guid, 16);
     ipmi_cmdlang_out(cmd_info, "Type",
 		    ipmi_domain_get_type_string(ipmi_domain_get_type(domain)));
     ipmi_cmdlang_out_int(cmd_info, "SEL Rescan Time",
@@ -366,6 +366,8 @@ domain_open(ipmi_cmd_info_t *cmd_info)
 	if (rv) {
 	    cmdlang->errstr = "Unable to setup connection";
 	    cmdlang->err = rv;
+	    for (j=0; j<i; j++)
+		con[j]->close_connection(con[j]);
 	    for (j=0; j<set; j++)
 		ipmi_free_args(con_parms[j]);
 	    goto out;
@@ -402,6 +404,19 @@ domain_open(ipmi_cmd_info_t *cmd_info)
     return;
 }
 
+void con_usage(const char *name, const char *help, void *cb_data)
+{
+    ipmi_cmdlang_t *cmdlang = cb_data;
+
+    cmdlang->out(cmdlang, name, help);
+}
+
+static void
+domain_open_help(ipmi_cmdlang_t *cmdlang)
+{
+    ipmi_parse_args_iter_help(con_usage, cmdlang);
+    cmdlang->out(cmdlang, "Options are:\n", ipmi_parse_options_help());
+}
 
 static void
 domain_fru_fetched(ipmi_domain_t *domain, ipmi_fru_t *fru,
@@ -547,7 +562,8 @@ domain_msg_handler(ipmi_domain_t *domain, ipmi_msgi_t *rspi)
     ipmi_cmdlang_out_int(cmd_info, "NetFN", msg->netfn);
     ipmi_cmdlang_out_int(cmd_info, "command", msg->cmd);
     if (msg->data_len)
-	ipmi_cmdlang_out_binary(cmd_info, "Data", msg->data, msg->data_len);
+	ipmi_cmdlang_out_binary(cmd_info, "Data",
+				(char *) msg->data, msg->data_len);
     ipmi_cmdlang_unlock(cmd_info);
     ipmi_cmdlang_up(cmd_info);
 
@@ -1071,6 +1087,12 @@ domain_con_change(ipmi_domain_t *domain,
     ipmi_cmdlang_out_bool(evi, "Any Connection Up", still_connected);
     ipmi_cmdlang_out_int(evi, "Error", err);
 
+    if (err) {
+	char errval[128];
+	ipmi_cmdlang_out(evi, "Error String",
+			 ipmi_get_error_string(err, errval, sizeof(errval)));
+	
+    }
     errstr = NULL; /* Get rid of warning */
 
  out_err:
@@ -1195,8 +1217,13 @@ ipmi_cmdlang_event_out(ipmi_event_t    *event,
 			  ipmi_event_get_timestamp(event));
     len = ipmi_event_get_data_len(event);
     if (len) {
-	ipmi_cmdlang_out_binary(cmd_info, "Data",
-				ipmi_event_get_data_ptr(event), len);
+	unsigned char *data;
+	data = ipmi_mem_alloc(len);
+	if (!data)
+	    return;
+	len = ipmi_event_get_data(event, data, 0, len);
+	ipmi_cmdlang_out_binary(cmd_info, "Data", (char *) data, len);
+	ipmi_mem_free(data);
     }
 }
 
@@ -1214,64 +1241,15 @@ static ipmi_cmdlang_init_t cmds_domain[] =
       "<domain> - Dump information about a domain",
       ipmi_cmdlang_domain_handler, domain_info, NULL },
     { "new", &domain_cmds,
-      "<domain name> [<options>] <domain parms> - Set up a new domain."
-      "  Options enable and disable various automatic processing and are:\n"
-      "-[no]all - all automatic handling\n"
-      "-[no]sdrs - sdr fetching\n"
-      "-[no]frus - FRU fetching\n"
-      "-[no]sel - SEL fetching\n"
-      "-[no]ipmbscan - IPMB bus scanning\n"
-      "-[no]oeminit - special OEM processing (like ATCA)\n"
-      "-[no]seteventrcvr - setting event receivers\n"
-      "-[no]activate - connection activation\n"
-      "-[no]setseltime - setting time of the SEL\n"
-      "-wait_til_up - wait until the domain is up before returning",
+      "Obsolete, use domain open",
       domain_new, NULL, NULL },
     { "open", &domain_cmds,
-      "<domain name> [<options>] <domain parms> - Set up a new domain using."
-      " a different style argument parser.  Format for SMI connections is:\n"
-      " smi <num>\n"
-      "where the number is the IPMI device number to connect to.  LAN parms"
-      " are:"
-      " lan [-U <username>] [-P <password>] [-p[2] port] [-A <authtype>]\n"
-      "     [-L <privilege>] [-s] <host1> [<host2>]\n"
-      "     [-Ra <auth alg>] [-Ri <integ alg>] [-Rc <conf algo>]\n"
-      "     [-Rl] [-Rk <bmc key>] [-H <hackname>]\n"
-      " If -s is supplied, then two host names are taken (the second port"
-      " may be specified with -p2).  Otherwise, only one hostname is"
-      " taken.  The defaults are an empty username and password (anonyous),"
-      " port 623, admin privilege, and authtype defaulting to the most"
-      " secure one available.\n"
-      "For RMCP+ connections, the authentication algorithms supported (-Ra)"
-      " are: bmcpick, rakp_none, rakp_hmac_sha1, and rakp_hmac_md5.  The"
-      " integrity algorithms (-Ri) supported are: bmcpick, none, hmac_sha1,"
-      " hmac_md5, and md5.  The confidentiality algorithms (-Rc) are: bmcpick,"
-      " aes_cbc_128, xrc4_128, and xrc_40.    The defaults are"
-      " rackp_hmac_sha1, hmac_sha1, and aes_cb_128.  -Rl turns on lookup up"
-      " names by"
-      " the name and the privilege level (allowing the same name with"
-      " different privileges and different passwords), the default is straight"
-      " name lookup.  -Rk sets the BMC key, needed if the system does two-key"
-      " lookups.\n"
-      " The -H option enables certain hacks for broken platforms.  This may"
-      " be listed multiple times to enable multiple hacks.  The currently"
-      " available hacks are:\n"
-      "   intelplus - For Intel platforms that have broken RMCP+.\n"
-      "   rakp3_wrong_rolem - For systems that truncate role(m) in the RAKP3"
-      " msg.\n"
-      "   rmcpp_integ_sik - For systems that use SIK instead of K(1) for"
-      " integrity.\n"
-      " Options enable and disable various automatic processing and are:\n"
-      "-[no]all - all automatic handling\n"
-      "-[no]sdrs - sdr fetching\n"
-      "-[no]frus - FRU fetching\n"
-      "-[no]sel - SEL fetching\n"
-      "-[no]ipmbscan - IPMB bus scanning\n"
-      "-[no]oeminit - special OEM processing (like ATCA)\n"
-      "-[no]seteventrcvr - setting event receivers\n"
-      "-[no]activate - connection activation\n"
-      "-wait_til_up - wait until the domain is up before returning",
-      domain_open, NULL, NULL },
+      "<domain name> [<options>] <domain parms> [<domain parms>]- Set up a"
+      " new domain using an argument parser.  Format for the connection's"
+      " <domain parms> depends on the connections type.  Two connections"
+      " (to two different MCs) can be done by specifying two sets of parms."
+      " Connections types are:",
+      domain_open, NULL, NULL, domain_open_help },
     { "close", &domain_cmds,
       "<domain> - Close the domain",
       ipmi_cmdlang_domain_handler, domain_close, NULL },
