@@ -50,7 +50,7 @@
 #include <OpenIPMI/ipmi_tcl.h>
 #include <OpenIPMI/ipmi_cmdlang.h>
 #include <OpenIPMI/ipmi_debug.h>
-#include <editline/readline.h>
+#include <readline/readline.h>
 
 #ifdef HAVE_GLIB
 #include <glib.h>
@@ -1049,6 +1049,8 @@ main(int argc, char *argv[])
 	}
     }
 
+    rl_initialize();
+
     if (use_debug_os) {
 	os_hnd = &ipmi_debug_os_handlers;
 	rv = sel_alloc_selector_nothread(&debug_sel);
