@@ -31,11 +31,12 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifndef __IPMI_POSIX_H
-#define __IPMI_POSIX_H
+#ifndef OPENIPMI_POSIX_H
+#define OPENIPMI_POSIX_H
 
 #include <OpenIPMI/os_handler.h>
 #include <OpenIPMI/selector.h>
+#include <OpenIPMI/deprecator.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -64,7 +65,7 @@ void posix_vlog(char *format,
 /* Allocate and configure an OS handler. */
 os_handler_t *ipmi_posix_setup_os_handler(void);
 /* Gets the selector associated with the OS handler. */
-selector_t *ipmi_posix_os_handler_get_sel(os_handler_t *os_hnd);
+struct selector_s *ipmi_posix_os_handler_get_sel(os_handler_t *os_hnd);
 
 
 /**********************************************************************
@@ -77,7 +78,7 @@ selector_t *ipmi_posix_os_handler_get_sel(os_handler_t *os_hnd);
  *	exit(1);
  *  }
  *
- *  rv = sel_alloc_selector(os_hnd, &sel);
+ *  rv = sel_alloc_selector_nothread(&sel);
  *  if (rv)
  *      handle_error();
  *
@@ -91,7 +92,8 @@ os_handler_t *ipmi_posix_get_os_handler(void);
 void ipmi_posix_free_os_handler(os_handler_t *os_hnd);
 /* You MUST set the SEL you alloc in the OS handler before you do
    anything else with the OS handler. */
-void ipmi_posix_os_handler_set_sel(os_handler_t *os_hnd, selector_t *sel);
+void ipmi_posix_os_handler_set_sel(os_handler_t *os_hnd,
+				   struct selector_s *sel);
 
 
 /**********************************************************************
@@ -105,16 +107,19 @@ void ipmi_posix_os_handler_set_sel(os_handler_t *os_hnd, selector_t *sel);
    it. */
 os_handler_t *ipmi_posix_thread_setup_os_handler(int wake_sig);
 /* Gets the selector associated with the OS handler. */
-selector_t *ipmi_posix_thread_os_handler_get_sel(os_handler_t *os_hnd);
+struct selector_s *ipmi_posix_thread_os_handler_get_sel(os_handler_t *os_hnd);
 
 /**********************************************************************
  * Special code, like the previous non-threaded ones.  Only needed
- * if you have special selector needs.
+ * if you have special selector needs.  Don't use
+ * ipmi_posix_thread_get_os_handler(), it has no way to set the
+ * wake signal.
  *********************************************************************/
 os_handler_t *ipmi_posix_thread_get_os_handler(void);
+os_handler_t *ipmi_posix_thread_get_os_handler2(int wake_sig);
 void ipmi_posix_thread_free_os_handler(os_handler_t *os_hnd);
 void ipmi_posix_thread_os_handler_set_sel(os_handler_t *os_hnd,
-					  selector_t   *sel);
+					  struct selector_s *sel);
 
 /**********************************************************************
  * Cruft, do not use these function any more.
@@ -160,4 +165,4 @@ void ipmi_posix_thread_sel_select_loop(os_handler_t *os_hnd)
 }
 #endif
 
-#endif /* __IPMI_POSIX_H */
+#endif /* OPENIPMI_POSIX_H */
