@@ -95,6 +95,11 @@ int sel_set_fd_handlers(struct selector_s *sel,
    sure to not be called again. */
 void sel_clear_fd_handlers(struct selector_s *sel,
 			   int        fd);
+/* Like above, but can only be called if no handlers are active.
+   If it detects a handler is active, it will assert.  This should
+   only be used at startup to clear the fd handler after an error
+   before any handlers are enabled. */
+void sel_clear_fd_handlers_imm(struct selector_s *sel, int fd);
 
 /* Turn on and off handling for I/O from a file descriptor. */
 #define SEL_FD_HANDLER_ENABLED	0
@@ -121,6 +126,11 @@ int sel_start_timer(sel_timer_t    *timer,
 		    struct timeval *timeout);
 
 int sel_stop_timer(sel_timer_t *timer);
+
+/* Stops the timer and calls the done handler when the stop is
+   complete.  This will return an error if the timer is not
+   running or if another done handler is pending running, and
+   the done handler will not be called. */
 int sel_stop_timer_with_done(sel_timer_t *timer,
 			     sel_timeout_handler_t done_handler,
 			     void *cb_data);
@@ -157,6 +167,15 @@ int sel_select(struct selector_s *sel,
 	       void            *cb_data,
 	       struct timeval  *timeout);
 
+/*
+ * Like the above call, but it will return EINTR if interrupted.
+ */
+int sel_select_intr(struct selector_s *sel,
+		    sel_send_sig_cb send_sig,
+		    long            thread_id,
+		    void            *cb_data,
+		    struct timeval  *timeout);
+
 /* This is the main loop for the program.  If NULL is passed in to
    send_sig, then the signal sender is not used.  If this encounters
    an unrecoverable problem with select(), it will return the errno.
@@ -166,6 +185,9 @@ int sel_select_loop(struct selector_s *sel,
 		    long            thread_id,
 		    void            *cb_data);
 
+/* Wake all threads in all select loops. */
+void sel_wake_all(struct selector_s *sel);
+
 typedef void (*ipmi_sel_add_read_fds_cb)(struct selector_s *sel,
 					 int            *num_fds,
 					 fd_set         *fdset,
