@@ -28,7 +28,24 @@
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this program; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+ *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * Modified BSD Licence
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *   1. Redistributions of source code must retain the above copyright
+ *      notice, this list of conditions and the following disclaimer.
+ *   2. Redistributions in binary form must reproduce the above
+ *      copyright notice, this list of conditions and the following
+ *      disclaimer in the documentation and/or other materials provided
+ *      with the distribution.
+ *   3. The name of the author may not be used to endorse or promote
+ *      products derived from this software without specific prior
+ *      written permission.
+ */
 
 #include "bmc.h"
 
@@ -1294,7 +1311,7 @@ file_init(lmc_data_t *mc,
 	  char **toks, void *cb_data, void **rcb_data,
 	  const char **errstr)
 {
-    const char *fname;
+    char *fname = NULL;
     struct file_data *f;
     char *end;
     int err;
@@ -1307,10 +1324,12 @@ file_init(lmc_data_t *mc,
 
     err = get_delim_str(toks, &fname, errstr);
     if (err)
-	return err;
-    f = malloc(sizeof(*f));
-    if (!f)
 	return ENOMEM;
+    f = malloc(sizeof(*f));
+    if (!f) {
+	free(fname);
+	return ENOMEM;
+    }
     memset(f, 0, sizeof(*f));
     f->emu = mc->emu;
     f->sensor_mc = mc;
@@ -1423,16 +1442,13 @@ file_init(lmc_data_t *mc,
 	tok = mystrtok(NULL, " \t\n", toks);
     }
 
-    f->filename = strdup(fname);
-    if (!f->filename) {
-	free(f);
-	return ENOMEM;
-    }
+    f->filename = fname;
 
     *rcb_data = f;
     return 0;
 
   out_err:
+    free(fname);
     free(f);
     return -1;
 }
@@ -1514,9 +1530,7 @@ sensor_poll(void *cb_data)
 	}
 	
 	if (sensor->event_reading_code == IPMI_EVENT_READING_TYPE_THRESHOLD) {
-	    if (val < 0)
-		val = 0;
-	    else if (val > 255)
+	    if (val > 255)
 		val = 255;
 	    set_sensor_value(mc, sensor, val, 1);
 	} else {
