@@ -715,6 +715,15 @@ static struct poptOption poptOpts[]=
 	"nostdio",
 	""
     },
+    {
+	"nopersist",
+	'p',
+	POPT_ARG_NONE,
+	NULL,
+	'p',
+	"nopersist",
+	""
+    },
     POPT_AUTOHELP
     {
 	NULL,
@@ -1376,6 +1385,9 @@ main(int argc, const char *argv[])
 	    case 'v':
 		print_version = 1;
 		break;
+	    case 'p':
+		persist_enable = 0;
+		break;
 	}
     }
     poptFreeContext(poptCtx);
