@@ -33,6 +33,7 @@
 
 #include <string.h>
 #include <stdio.h>
+#include <limits.h>
 
 #include <OpenIPMI/ipmiif.h>
 #include <OpenIPMI/ipmi_err.h>
@@ -102,11 +103,12 @@ struct ipmi_control_s
 
     int has_events;
 
-    int ignore_if_no_entity;
+    int ignore_if_no_entity : 1;
+    int ignore_for_presence : 1;
 
     /* A list of handlers to call when an event for the control comes
        in. */
-    locked_list_t *handler_list;
+    locked_list_t *handler_list, *handler_list_cl;
 
     /* For light types. */
 #define MAX_LIGHTS 10
@@ -150,7 +152,7 @@ static void control_final_destroy(ipmi_control_t *control);
 
 /* Must be called with the domain entity lock held. */
 int
-_ipmi_control_get(ipmi_control_t *control)
+i_ipmi_control_get(ipmi_control_t *control)
 {
     if (control->destroyed)
 	return EINVAL;
@@ -159,28 +161,28 @@ _ipmi_control_get(ipmi_control_t *control)
 }
 
 void
-_ipmi_control_put(ipmi_control_t *control)
+i_ipmi_control_put(ipmi_control_t *control)
 {
-    _ipmi_domain_entity_lock(control->domain);
+    i_ipmi_domain_entity_lock(control->domain);
     if (control->usecount == 1) {
 	if (control->add_pending) {
 	    control->add_pending = 0;
-	    _ipmi_domain_entity_unlock(control->domain);
-	    _ipmi_entity_call_control_handlers(control->entity,
+	    i_ipmi_domain_entity_unlock(control->domain);
+	    i_ipmi_entity_call_control_handlers(control->entity,
 					       control, IPMI_ADDED);
-	    _ipmi_domain_entity_lock(control->domain);
+	    i_ipmi_domain_entity_lock(control->domain);
 	}
 	if (control->destroyed
 	    && (!control->waitq
 		|| (!opq_stuff_in_progress(control->waitq))))
 	{
-	    _ipmi_domain_entity_unlock(control->domain);
+	    i_ipmi_domain_entity_unlock(control->domain);
 	    control_final_destroy(control);
 	    return;
 	}
     }
     control->usecount--;
-    _ipmi_domain_entity_unlock(control->domain);
+    i_ipmi_domain_entity_unlock(control->domain);
 }
 
 ipmi_control_id_t
@@ -214,8 +216,8 @@ mc_cb(ipmi_mc_t *mc, void *cb_data)
     ipmi_control_t      *control;
     ipmi_entity_t       *entity = NULL;
     
-    controls = _ipmi_mc_get_controls(mc);
-    _ipmi_domain_entity_lock(domain);
+    controls = i_ipmi_mc_get_controls(mc);
+    i_ipmi_domain_entity_lock(domain);
     if (info->id.lun > 4) {
 	info->err = EINVAL;
 	goto out_unlock;
@@ -232,27 +234,27 @@ mc_cb(ipmi_mc_t *mc, void *cb_data)
 	goto out_unlock;
     }
 
-    info->err = _ipmi_entity_get(control->entity);
+    info->err = i_ipmi_entity_get(control->entity);
     if (info->err)
 	goto out_unlock;
     entity = control->entity;
 
-    info->err = _ipmi_control_get(control);
+    info->err = i_ipmi_control_get(control);
     if (info->err)
 	goto out_unlock;
 
-    _ipmi_domain_entity_unlock(domain);
+    i_ipmi_domain_entity_unlock(domain);
 
     info->handler(control, info->cb_data);
 
-    _ipmi_control_put(control);
-    _ipmi_entity_put(entity);
+    i_ipmi_control_put(control);
+    i_ipmi_entity_put(entity);
     return;
 
  out_unlock:
-    _ipmi_domain_entity_unlock(domain);
+    i_ipmi_domain_entity_unlock(domain);
     if (entity)
-	_ipmi_entity_put(entity);
+	i_ipmi_entity_put(entity);
 }
 
 int
@@ -372,14 +374,44 @@ static int
 control_ok_to_use(ipmi_control_t *control)
 {
     return (   !control->destroyed
-	    && !_ipmi_domain_in_shutdown(control->domain));
+	    && !i_ipmi_domain_in_shutdown(control->domain));
+}
+
+typedef struct handler_cl_info_s
+{
+    ipmi_control_val_event_cb handler;
+    void                      *handler_data;
+} handler_cl_info_t;
+
+static int
+iterate_handler_cl(void *cb_data, void *item1, void *item2)
+{
+    handler_cl_info_t            *info = cb_data;
+    ipmi_control_val_event_cl_cb handler = item1;
+
+    handler(info->handler, info->handler_data, item2);
+    return LOCKED_LIST_ITER_CONTINUE;
+}
+
+static int
+handler_list_cleanup(void *cb_data, void *item1, void *item2)
+{
+    ipmi_control_t    *control = cb_data;
+    handler_cl_info_t info;
+
+    info.handler = item1;
+    info.handler_data = item2;
+    locked_list_iterate(control->handler_list_cl,
+			iterate_handler_cl,
+			&info);
+    return LOCKED_LIST_ITER_CONTINUE;
 }
 
 static void
 control_final_destroy(ipmi_control_t *control)
 {
-    _ipmi_entity_get(control->entity);
-    _ipmi_entity_call_control_handlers(control->entity, control, IPMI_DELETED);
+    i_ipmi_entity_get(control->entity);
+    i_ipmi_entity_call_control_handlers(control->entity, control, IPMI_DELETED);
 
     control->mc = NULL;
 
@@ -387,11 +419,14 @@ control_final_destroy(ipmi_control_t *control)
 	control->destroy_handler(control,
 				 control->destroy_handler_cb_data);
 
-    if (control->oem_info_cleanup_handler)
-	control->oem_info_cleanup_handler(control, control->oem_info);
-
-    if (control->handler_list)
+    if (control->handler_list) {
+	locked_list_iterate(control->handler_list_cl, handler_list_cleanup,
+			    control);
 	locked_list_destroy(control->handler_list);
+    }
+
+    if (control->handler_list_cl)
+	locked_list_destroy(control->handler_list_cl);
 
     if (control->waitq)
 	opq_destroy(control->waitq);
@@ -399,7 +434,10 @@ control_final_destroy(ipmi_control_t *control)
     if (control->entity)
 	ipmi_entity_remove_control(control->entity, control);
 
-    _ipmi_entity_put(control->entity);
+    if (control->oem_info_cleanup_handler)
+	control->oem_info_cleanup_handler(control, control->oem_info);
+
+    i_ipmi_entity_put(control->entity);
     ipmi_mem_free(control);
 }
 
@@ -409,28 +447,24 @@ ipmi_control_destroy(ipmi_control_t *control)
     ipmi_control_info_t *controls;
     ipmi_mc_t           *mc = control->mc;
 
-    _ipmi_domain_mc_lock(control->domain);
-    _ipmi_mc_get(mc);
-    _ipmi_domain_mc_unlock(control->domain);
-    controls = _ipmi_mc_get_controls(control->mc);
+    i_ipmi_domain_mc_lock(control->domain);
+    i_ipmi_mc_get(mc);
+    i_ipmi_domain_mc_unlock(control->domain);
+    controls = i_ipmi_mc_get_controls(control->mc);
 
     ipmi_lock(controls->idx_lock);
-    if (controls->controls_by_idx[control->num] != control) {
-	ipmi_unlock(controls->idx_lock);
-	_ipmi_mc_put(control->mc);
-	return EINVAL;
+    if (controls->controls_by_idx[control->num] == control) {
+	controls->control_count--;
+	controls->controls_by_idx[control->num] = NULL;
     }
 
-    _ipmi_control_get(control);
-
-    controls->control_count--;
-    controls->controls_by_idx[control->num] = NULL;
+    i_ipmi_control_get(control);
 
     ipmi_unlock(controls->idx_lock);
 
     control->destroyed = 1;
-    _ipmi_control_put(control);
-    _ipmi_mc_put(mc);
+    i_ipmi_control_put(control);
+    i_ipmi_mc_put(mc);
 
     return 0;
 }
@@ -453,7 +487,7 @@ control_set_name(ipmi_control_t *control)
 }
 
 const char *
-_ipmi_control_name(const ipmi_control_t *control)
+i_ipmi_control_name(const ipmi_control_t *control)
 {
     return control->name;
 }
@@ -572,11 +606,11 @@ control_rsp_handler(ipmi_mc_t  *mc,
     if (control->destroyed) {
 	ipmi_entity_t *entity = NULL;
 
-	_ipmi_domain_entity_lock(control->domain);
+	i_ipmi_domain_entity_lock(control->domain);
 	control->usecount++;
-	_ipmi_domain_entity_unlock(control->domain);
+	i_ipmi_domain_entity_unlock(control->domain);
 
-	rv = _ipmi_entity_get(control->entity);
+	rv = i_ipmi_entity_get(control->entity);
 	if (! rv)
 	    entity = control->entity;
 
@@ -587,9 +621,9 @@ control_rsp_handler(ipmi_mc_t  *mc,
 	if (info->__rsp_handler)
 	    info->__rsp_handler(control, ECANCELED, NULL, info->__cb_data);
 
-	_ipmi_control_put(control);
+	i_ipmi_control_put(control);
 	if (entity)
-	    _ipmi_entity_put(entity);
+	    i_ipmi_entity_put(entity);
 	return;
     }
 
@@ -598,20 +632,20 @@ control_rsp_handler(ipmi_mc_t  *mc,
 		 "control.c(control_rsp_handler): "
 		 "MC was destroyed while a control operation was in progress");
 
-	_ipmi_domain_entity_lock(control->domain);
+	i_ipmi_domain_entity_lock(control->domain);
 	control->usecount++;
-	_ipmi_domain_entity_unlock(control->domain);
+	i_ipmi_domain_entity_unlock(control->domain);
 
-	rv = _ipmi_entity_get(control->entity);
+	rv = i_ipmi_entity_get(control->entity);
 	if (! rv)
 	    entity = control->entity;
 
 	if (info->__rsp_handler)
 	    info->__rsp_handler(control, ECANCELED, NULL, info->__cb_data);
 
-	_ipmi_control_put(control);
+	i_ipmi_control_put(control);
 	if (entity)
-	    _ipmi_entity_put(entity);
+	    i_ipmi_entity_put(entity);
 
 	return;
     }
@@ -628,20 +662,20 @@ control_rsp_handler(ipmi_mc_t  *mc,
 		 "%scontrol.c(control_rsp_handler): "
 		 "Could not convert control id to a pointer",
 		 MC_NAME(mc));
-	_ipmi_domain_entity_lock(control->domain);
+	i_ipmi_domain_entity_lock(control->domain);
 	control->usecount++;
-	_ipmi_domain_entity_unlock(control->domain);
+	i_ipmi_domain_entity_unlock(control->domain);
 
-	nrv = _ipmi_entity_get(control->entity);
+	nrv = i_ipmi_entity_get(control->entity);
 	if (! nrv)
 	    entity = control->entity;
 
 	if (info->__rsp_handler)
 	    info->__rsp_handler(control, rv, NULL, info->__cb_data);
 
-	_ipmi_control_put(control);
+	i_ipmi_control_put(control);
 	if (entity)
-	    _ipmi_entity_put(entity);
+	    i_ipmi_entity_put(entity);
     }
 }
 			 
@@ -686,10 +720,10 @@ control_addr_response_handler(ipmi_domain_t *domain, ipmi_msgi_t *rspi)
 	if (info->__rsp_handler)
 	    info->__rsp_handler(control, ECANCELED, NULL, info->__cb_data);
 
-	_ipmi_domain_entity_lock(control->domain);
+	i_ipmi_domain_entity_lock(control->domain);
 	control->usecount++;
-	_ipmi_domain_entity_unlock(control->domain);
-	_ipmi_control_put(control);
+	i_ipmi_domain_entity_unlock(control->domain);
+	i_ipmi_control_put(control);
 	return IPMI_MSG_ITEM_NOT_USED;
     }
 
@@ -704,11 +738,11 @@ control_addr_response_handler(ipmi_domain_t *domain, ipmi_msgi_t *rspi)
 		 "Could not convert control id to a pointer",
 		 DOMAIN_NAME(domain));
 	if (info->__rsp_handler) {
-	    _ipmi_domain_entity_lock(control->domain);
+	    i_ipmi_domain_entity_lock(control->domain);
 	    control->usecount++;
-	    _ipmi_domain_entity_unlock(control->domain);
+	    i_ipmi_domain_entity_unlock(control->domain);
 	    info->__rsp_handler(control, rv, NULL, info->__cb_data);
-	    _ipmi_control_put(control);
+	    i_ipmi_control_put(control);
 	}
     }
     return IPMI_MSG_ITEM_NOT_USED;
@@ -806,8 +840,10 @@ ipmi_control_add_nonstandard(ipmi_mc_t               *mc,
 {
     ipmi_domain_t       *domain;
     os_handler_t        *os_hnd;
-    ipmi_control_info_t *controls = _ipmi_mc_get_controls(mc);
+    ipmi_control_info_t *controls = i_ipmi_mc_get_controls(mc);
     locked_list_entry_t *link;
+    int                 err;
+    unsigned int        i;
 
     CHECK_MC_LOCK(mc);
     CHECK_ENTITY_LOCK(ent);
@@ -815,12 +851,24 @@ ipmi_control_add_nonstandard(ipmi_mc_t               *mc,
     domain = ipmi_mc_get_domain(mc);
     os_hnd = ipmi_domain_get_os_hnd(domain);
 
-    if (num >= 256)
+    if ((num >= 256) && (num != UINT_MAX))
 	return EINVAL;
 
-    _ipmi_domain_entity_lock(domain);
+    i_ipmi_domain_entity_lock(domain);
     ipmi_lock(controls->idx_lock);
 
+    if (num == UINT_MAX){
+	for (i=0; i<controls->idx_size; i++) {
+	    if (! controls->controls_by_idx[i])
+		break;
+	}
+	num = i;
+	if (num >= 256) {
+	    err = EAGAIN;
+	    goto out_err;
+	}
+    }
+
     if (num >= controls->idx_size) {
 	ipmi_control_t **new_array;
 	unsigned int   new_size;
@@ -831,8 +879,8 @@ ipmi_control_add_nonstandard(ipmi_mc_t               *mc,
 	new_size = ((num / 16) * 16) + 16;
 	new_array = ipmi_mem_alloc(sizeof(*new_array) * new_size);
 	if (!new_array) {
-	    ipmi_unlock(controls->idx_lock);
-	    return ENOMEM;
+	    err = ENOMEM;
+	    goto out_err;
 	}
 	if (controls->controls_by_idx)
 	    memcpy(new_array, controls->controls_by_idx,
@@ -847,15 +895,23 @@ ipmi_control_add_nonstandard(ipmi_mc_t               *mc,
 
     control->waitq = opq_alloc(os_hnd);
     if (! control->waitq) {
-	ipmi_unlock(controls->idx_lock);
-	return ENOMEM;
+	err = ENOMEM;
+	goto out_err;
+    }
+
+    control->handler_list_cl = locked_list_alloc(os_hnd);
+    if (! control->handler_list_cl) {
+	opq_destroy(control->waitq);
+	err = ENOMEM;
+	goto out_err;
     }
 
     control->handler_list = locked_list_alloc(os_hnd);
     if (! control->handler_list) {
 	opq_destroy(control->waitq);
-	ipmi_unlock(controls->idx_lock);
-	return ENOMEM;
+	locked_list_destroy(control->handler_list_cl);
+	err = ENOMEM;
+	goto out_err;
     }
 
     link = locked_list_alloc_entry();
@@ -863,9 +919,10 @@ ipmi_control_add_nonstandard(ipmi_mc_t               *mc,
 	opq_destroy(control->waitq);
 	control->waitq = NULL;
 	locked_list_destroy(control->handler_list);
+	locked_list_destroy(control->handler_list_cl);
 	control->handler_list = NULL;
-	ipmi_unlock(controls->idx_lock);
-	return ENOMEM;
+	err = ENOMEM;
+	goto out_err;
     }
 
     control->domain = domain;
@@ -890,13 +947,18 @@ ipmi_control_add_nonstandard(ipmi_mc_t               *mc,
 
     ipmi_unlock(controls->idx_lock);
 
-    _ipmi_domain_entity_unlock(domain);
+    i_ipmi_domain_entity_unlock(domain);
 
     ipmi_entity_add_control(ent, control, link);
 
     control->add_pending = 1;
 
     return 0;
+
+ out_err:
+    ipmi_unlock(controls->idx_lock);
+    i_ipmi_domain_entity_unlock(domain);
+    return err;
 }
 
 int
@@ -1294,6 +1356,31 @@ int ipmi_control_remove_val_event_handler(ipmi_control_t            *control,
     return 0;
 }
 
+int
+ipmi_control_add_val_event_handler_cl(ipmi_control_t            *control,
+				      ipmi_control_val_event_cl_cb handler,
+				      void                      *cb_data)
+{
+    CHECK_CONTROL_LOCK(control);
+
+    if (! locked_list_add(control->handler_list_cl, handler, cb_data))
+	return ENOMEM;
+
+    return 0;
+}
+
+int ipmi_control_remove_val_event_handler_cl(ipmi_control_t         *control,
+					  ipmi_control_val_event_cl_cb handler,
+					  void                      *cb_data)
+{
+    CHECK_CONTROL_LOCK(control);
+
+    if (! locked_list_remove(control->handler_list_cl, handler, cb_data))
+	return ENOENT;
+
+    return 0;
+}
+
 /***********************************************************************
  *
  * Get/set various local information about a control.
@@ -1940,6 +2027,20 @@ ipmi_control_set_ignore_if_no_entity(ipmi_control_t *control,
     control->ignore_if_no_entity = ignore_if_no_entity;
 }
 
+int
+ipmi_control_get_ignore_for_presence(ipmi_control_t *control)
+{
+    CHECK_CONTROL_LOCK(control);
+
+    return control->ignore_for_presence;
+}
+
+void
+ipmi_control_set_ignore_for_presence(ipmi_control_t *control, int ignore)
+{
+    control->ignore_for_presence = ignore;
+}
+
 ipmi_domain_t *
 ipmi_control_get_domain(ipmi_control_t *control)
 {
@@ -1948,7 +2049,7 @@ ipmi_control_get_domain(ipmi_control_t *control)
 
 #ifdef IPMI_CHECK_LOCKS
 void
-__ipmi_check_control_lock(const ipmi_control_t *control)
+i__ipmi_check_control_lock(const ipmi_control_t *control)
 {
     if (!control)
 	return;
