@@ -53,6 +53,18 @@ ipmi_update_e_string(enum ipmi_update_e val)
     }
 }
 
+const char *
+ipmi_update_werr_e_string(enum ipmi_update_werr_e val)
+{
+    switch (val) {
+    case IPMIE_ADDED: return "added";
+    case IPMIE_DELETED: return "deleted";
+    case IPMIE_CHANGED: return "changed";
+    case IPMIE_ERROR: return "error";
+    default: return "invalid";
+    }
+}
+
 static char *hysteresis_support_types[] =
 {
     "none",
@@ -140,7 +152,7 @@ static char *sensor_types[] =
     "system_acpi_power_state",
     "watchdog_2",
     "platform_alert",
-    "entity_presense",
+    "entity_presence",
     "monitor_asic_ic",
     "lan",
     "management_subsystem_health",
@@ -168,7 +180,7 @@ static char *event_reading_types[] =
     "discrete_limit_exceeded",
     "discrete_performance_met",
     "discrete_severity",
-    "discrete_device_presense",
+    "discrete_device_presence",
     "discrete_device_enable",
     "discrete_availability",
     "discrete_redundancy",
@@ -338,7 +350,7 @@ static char *threshold_types[] =
 const char *
 ipmi_get_threshold_string(enum ipmi_thresh_e val)
 {
-    if (val > NUM_THRESHOLD_TYPES)
+    if (val >= NUM_THRESHOLD_TYPES)
 	return "invalid";
     return threshold_types[val];
 }
@@ -352,7 +364,7 @@ static char *value_dir_types[] =
 const char *
 ipmi_get_value_dir_string(enum ipmi_event_value_dir_e val)
 {
-    if (val > NUM_VALUE_DIR_TYPES)
+    if (val >= NUM_VALUE_DIR_TYPES)
 	return "invalid";
     return value_dir_types[val];
 }
@@ -366,7 +378,7 @@ static char *event_dir_types[] =
 const char *
 ipmi_get_event_dir_string(enum ipmi_event_dir_e val)
 {
-    if (val > NUM_EVENT_DIR_TYPES)
+    if (val >= NUM_EVENT_DIR_TYPES)
 	return "invalid";
     return event_dir_types[val];
 }
@@ -577,7 +589,7 @@ static char *sensor_states[256][16] =
     },
     { /* 0x10 */
 	"correctable memory error logging disabled",
-	"event type logging siabled",
+	"event type logging disabled",
 	"log area reset/cleared",
 	"all event logging disabled",
     },
