@@ -37,7 +37,6 @@
 #include <netinet/tcp.h>
 #include <netdb.h>
 #include <errno.h>
-#include <malloc.h>
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
@@ -48,18 +47,6 @@
 #define _GNU_SOURCE
 #include <getopt.h>
 
-typedef struct sockaddr_ip_s {
-    union
-        {
-	    struct sockaddr	s_addr;
-            struct sockaddr_in  s_addr4;
-#ifdef PF_INET6
-            struct sockaddr_in6 s_addr6;
-#endif
-        } s_ipsock;
-} sockaddr_ip_t;
-
-
 #define IPMB_MAX_MSG_LENGTH 32
 #define IPMI_MAX_MSG_LENGTH 36
 
@@ -113,7 +100,7 @@ struct msg_info {
     int           debug;
     int           do_attn;
     unsigned char attn_chars[8];
-    int           attn_chars_len;
+    unsigned int  attn_chars_len;
     char          global_enables;
 
     /* Info from the recv message. */
@@ -253,8 +240,8 @@ struct ra_data {
 static void ra_format_msg(const unsigned char *msg, unsigned int msg_len,
 			  struct msg_info *mi)
 {
-    int i;
-    int len;
+    unsigned int i;
+    unsigned int len;
     unsigned char c[RA_MAX_CHARS_SIZE];
 
     len = 0;
@@ -572,8 +559,8 @@ struct tm_data {
 static void tm_format_msg(const unsigned char *msg, unsigned int msg_len,
 			  struct msg_info *mi)
 {
-    int i;
-    int len;
+    unsigned int i;
+    unsigned int len;
     unsigned char c[TM_MAX_CHARS_SIZE];
     unsigned char t;
 
@@ -774,7 +761,7 @@ static void
 socket_send(const unsigned char *data, unsigned int len, struct msg_info *mi)
 {
     int rv;
-    int i;
+    unsigned int i;
 
     if (mi->debug > 0) {
 	printf("Sock send:");
@@ -791,7 +778,7 @@ socket_send(const unsigned char *data, unsigned int len, struct msg_info *mi)
     if (rv < 0) {
 	perror("write");
 	return;
-    } else if (rv < len) {
+    } else if (((unsigned int) rv) < len) {
 	len -= rv;
 	data += rv;
 	goto restart;
@@ -821,7 +808,7 @@ handle_ipmb_msg(const unsigned char *msg, unsigned int len,
     struct msg *imsg;
     unsigned char rsp[IPMI_MAX_MSG_LENGTH];
     unsigned int rsp_len;
-    int          i;
+    unsigned int i;
 
     imsg = malloc(sizeof(*imsg));
     if (!imsg)
@@ -876,7 +863,7 @@ static unsigned char guid_data[] = {
 static void
 handle_msg(const unsigned char *msg, unsigned int len, struct msg_info *mi)
 {
-    int i;
+    unsigned int i;
     unsigned char rsp[IPMI_MAX_MSG_LENGTH];
     unsigned int rsp_len;
     struct msg *m;
@@ -1327,9 +1314,11 @@ usage(void)
 int
 main(int argc, char *argv[])
 {
-    int i;
+    unsigned int i;
     struct addrinfo hints, *res0;
-    sockaddr_ip_t addr;
+    struct sockaddr_storage saddr;
+    struct sockaddr *addr = (struct sockaddr *) &saddr;
+    size_t addrlen;
     struct msg_info *mi = &main_mi;
     int rv;
     char *s, *e;
@@ -1415,29 +1404,31 @@ main(int argc, char *argv[])
     }
 
     i = optind;
-    if (i+2 < argc) {
+    if (i+2 < ((unsigned int) argc)) {
 	fprintf(stderr, "Host and/or port not supplied\n");
 	usage();
     }
 
     memset(&hints, 0, sizeof(hints));
     hints.ai_socktype = SOCK_STREAM;
+    hints.ai_family = AF_UNSPEC;
     rv = getaddrinfo(argv[i], argv[i+1], &hints, &res0);
     if (rv) {
 	perror("getaddrinfo");
 	usage();
     }
     /* Only get the first choices */
-    memcpy(&addr, res0->ai_addr, res0->ai_addrlen);
+    memcpy(addr, res0->ai_addr, res0->ai_addrlen);
+    addrlen = res0->ai_addrlen;
     freeaddrinfo(res0);
 
-    mi->sock = socket(PF_INET, SOCK_STREAM, 0);
+    mi->sock = socket(addr->sa_family, SOCK_STREAM, 0);
     if (mi->sock < 0) {
 	perror("socket");
 	usage();
     }
 
-    rv = connect(mi->sock, (struct sockaddr *) &addr, sizeof(addr));
+    rv = connect(mi->sock, (struct sockaddr *) addr, addrlen);
     if (rv < 0) {
 	perror("connect");
 	usage();
@@ -1472,6 +1463,7 @@ main(int argc, char *argv[])
 	unsigned char buf[128];
 	int i;
 	fd_set readfds;
+	int rv2;
 
 	FD_ZERO(&readfds);
 	FD_SET(0, &readfds);
@@ -1508,8 +1500,13 @@ main(int argc, char *argv[])
 		 * Echo one at a time in case the echo gets turned off
 		 * in the middle of this data.
 		 */
-		if (mi->echo)
-		    write(mi->sock, buf+i, 1);
+		if (mi->echo) {
+		    rv2 = write(mi->sock, buf+i, 1);
+		    if (rv2 < 0) {
+			perror("write");
+			usage();
+		    }
+		}
 		mi->codec->handle_char(buf[i], mi);
 	    }
 	}
