@@ -64,9 +64,25 @@ extern void posix_vlog(char                 *format,
 		       va_list              ap);
 #pragma weak posix_vlog
 
+static void i_posix_lock(pthread_mutex_t *lock)
+{
+    int rv = pthread_mutex_lock(lock);
+
+    if (rv)
+	abort();
+}
+
+static void i_posix_unlock(pthread_mutex_t *lock)
+{
+    int rv = pthread_mutex_unlock(lock);
+
+    if (rv)
+	abort();
+}
+
 typedef struct pt_os_hnd_data_s
 {
-    selector_t       *sel;
+    struct selector_s *sel;
     os_vlog_t        log_handler;
     int              wake_sig;
     struct sigaction oldact;
@@ -120,7 +136,7 @@ add_fd(os_handler_t       *handler,
     os_hnd_fd_id_t   *fd_data;
     int              rv;
     pt_os_hnd_data_t *info = handler->internal_data;
-    selector_t       *posix_sel = info->sel;
+    struct selector_s *posix_sel = info->sel;
 
     fd_data = malloc(sizeof(*fd_data));
     if (!fd_data)
@@ -149,7 +165,7 @@ static int
 remove_fd(os_handler_t *handler, os_hnd_fd_id_t *fd_data)
 {
     pt_os_hnd_data_t *info = handler->internal_data;
-    selector_t       *posix_sel = info->sel;
+    struct selector_s *posix_sel = info->sel;
 
     sel_set_fd_read_handler(posix_sel, fd_data->fd, SEL_FD_HANDLER_DISABLED);
     sel_clear_fd_handlers(posix_sel, fd_data->fd);
@@ -169,7 +185,7 @@ struct os_hnd_timer_id_s
 };
 
 static void
-timer_handler(selector_t  *sel,
+timer_handler(struct selector_s *sel,
 	      sel_timer_t *timer,
 	      void        *data)
 {
@@ -180,11 +196,11 @@ timer_handler(selector_t  *sel,
     os_timed_out_t    timed_out;
 
     /* Lock will create a memory barrier, which we really need here. */
-    pthread_mutex_lock(&timer_data->lock);
+    i_posix_lock(&timer_data->lock);
     timed_out = timer_data->timed_out;
     cb_data = timer_data->cb_data;
     timer_data->running = 0;
-    pthread_mutex_unlock(&timer_data->lock);
+    i_posix_unlock(&timer_data->lock);
     timed_out(cb_data, timer_data);
 }
 
@@ -198,7 +214,7 @@ start_timer(os_handler_t      *handler,
     struct timeval now;
     int rv;
 
-    pthread_mutex_lock(&id->lock);
+    i_posix_lock(&id->lock);
     if (id->running) {
 	rv = EBUSY;
 	goto out;
@@ -223,7 +239,7 @@ start_timer(os_handler_t      *handler,
     if (rv != 0)
 	id->running = 0;
   out:
-    pthread_mutex_unlock(&id->lock);
+    i_posix_unlock(&id->lock);
     return rv;
 }
 
@@ -232,11 +248,11 @@ stop_timer(os_handler_t *handler, os_hnd_timer_id_t *timer_data)
 {
     int rv;
 
-    pthread_mutex_lock(&timer_data->lock);
+    i_posix_lock(&timer_data->lock);
     rv = sel_stop_timer(timer_data->timer);
     if (rv == 0)
 	timer_data->running = 0;
-    pthread_mutex_unlock(&timer_data->lock);
+    i_posix_unlock(&timer_data->lock);
     return rv;
 }
 
@@ -247,7 +263,7 @@ alloc_timer(os_handler_t      *handler,
     os_hnd_timer_id_t *timer_data;
     int               rv;
     pt_os_hnd_data_t  *info = handler->internal_data;
-    selector_t        *posix_sel = info->sel;
+    struct selector_s *posix_sel = info->sel;
 
     timer_data = malloc(sizeof(*timer_data));
     if (!timer_data)
@@ -442,11 +458,7 @@ static int
 lock(os_handler_t  *handler,
      os_hnd_lock_t *id)
 {
-    int rv;
-
-    rv = pthread_mutex_lock(&id->mutex);
-    if (rv)
-	return rv;
+    i_posix_lock(&id->mutex);
     return 0;
 }
 
@@ -454,11 +466,7 @@ static int
 unlock(os_handler_t  *handler,
        os_hnd_lock_t *id)
 {
-    int rv;
-
-    rv = pthread_mutex_unlock(&id->mutex);
-    if (rv)
-	return rv;
+    i_posix_unlock(&id->mutex);
     return 0;
 }
 
@@ -624,14 +632,15 @@ ipmi_posix_thread_free_os_handler(os_handler_t *os_hnd)
 }
 
 void
-ipmi_posix_thread_os_handler_set_sel(os_handler_t *os_hnd, selector_t *sel)
+ipmi_posix_thread_os_handler_set_sel(os_handler_t *os_hnd,
+				     struct selector_s *sel)
 {
     pt_os_hnd_data_t  *info = os_hnd->internal_data;
 
     info->sel = sel;
 }
 
-selector_t *
+struct selector_s *
 ipmi_posix_thread_os_handler_get_sel(os_handler_t *os_hnd)
 {
     pt_os_hnd_data_t  *info = os_hnd->internal_data;
@@ -733,11 +742,11 @@ database_store(os_handler_t  *handler,
     datum            gkey, gdata;
     int              rv;
 
-    pthread_mutex_lock(&info->gdbm_lock);
+    i_posix_lock(&info->gdbm_lock);
     if (!info->gdbmf) {
 	init_gdbm(info);
 	if (!info->gdbmf) {
-	    pthread_mutex_unlock(&info->gdbm_lock);
+	    i_posix_unlock(&info->gdbm_lock);
 	    return EINVAL;
 	}
     }
@@ -748,7 +757,7 @@ database_store(os_handler_t  *handler,
     gdata.dsize = data_len;
 
     rv = gdbm_store(info->gdbmf, gkey, gdata, GDBM_REPLACE);
-    pthread_mutex_unlock(&info->gdbm_lock);
+    i_posix_unlock(&info->gdbm_lock);
     if (rv)
 	return EINVAL;
     return 0;
@@ -769,11 +778,11 @@ database_find(os_handler_t  *handler,
     pt_os_hnd_data_t *info = handler->internal_data;
     datum            gkey, gdata;
 
-    pthread_mutex_lock(&info->gdbm_lock);
+    i_posix_lock(&info->gdbm_lock);
     if (!info->gdbmf) {
 	init_gdbm(info);
 	if (!info->gdbmf) {
-	    pthread_mutex_unlock(&info->gdbm_lock);
+	    i_posix_unlock(&info->gdbm_lock);
 	    return EINVAL;
 	}
     }
@@ -781,7 +790,7 @@ database_find(os_handler_t  *handler,
     gkey.dptr = key;
     gkey.dsize = strlen(key);
     gdata = gdbm_fetch(info->gdbmf, gkey);
-    pthread_mutex_unlock(&info->gdbm_lock);
+    i_posix_unlock(&info->gdbm_lock);
     if (!gdata.dptr)
 	return EINVAL;
     *data = (unsigned char *) gdata.dptr;
@@ -887,7 +896,7 @@ static os_handler_t ipmi_posix_thread_os_handler =
 };
 
 os_handler_t *
-ipmi_posix_thread_get_os_handler(void)
+ipmi_posix_thread_get_os_handler2(int wake_sig)
 {
     os_handler_t     *rv;
     pt_os_hnd_data_t *info;
@@ -918,9 +927,59 @@ ipmi_posix_thread_get_os_handler(void)
     }
 #endif
 
+    info->wake_sig = wake_sig;
+
     return rv;
 }
 
+os_handler_t *
+ipmi_posix_thread_get_os_handler(void)
+{
+    return ipmi_posix_thread_get_os_handler2(0);
+}
+
+struct sel_lock_s
+{
+    os_handler_t *os_hnd;
+    os_hnd_lock_t *lock;
+};
+
+static sel_lock_t *
+slock_alloc(void *cb_data)
+{
+    os_handler_t *os_hnd = cb_data;
+    sel_lock_t *l;
+
+    l = os_hnd->mem_alloc(sizeof(*l));
+    if (!l)
+	return NULL;
+    l->os_hnd = os_hnd;
+    if (os_hnd->create_lock(os_hnd, &l->lock)) {
+	os_hnd->mem_free(l);
+	l = NULL;
+    }
+    return l;
+}
+
+static void
+slock_free(sel_lock_t *l)
+{
+    l->os_hnd->destroy_lock(l->os_hnd, l->lock);
+    l->os_hnd->mem_free(l);
+}
+
+static void
+slock_lock(sel_lock_t *l)
+{
+    l->os_hnd->lock(l->os_hnd, l->lock);
+}
+
+static void
+slock_unlock(sel_lock_t *l)
+{
+    l->os_hnd->unlock(l->os_hnd, l->lock);
+}
+
 os_handler_t *
 ipmi_posix_thread_setup_os_handler(int wake_sig)
 {
@@ -929,14 +988,15 @@ ipmi_posix_thread_setup_os_handler(int wake_sig)
     struct sigaction act;
     int              rv;
 
-    os_hnd = ipmi_posix_thread_get_os_handler();
+    os_hnd = ipmi_posix_thread_get_os_handler2(wake_sig);
     if (!os_hnd)
 	return NULL;
 
     info = os_hnd->internal_data;
-    info->wake_sig = wake_sig;
 
-    rv = sel_alloc_selector(os_hnd, &info->sel);
+    rv = sel_alloc_selector_thread(&info->sel, wake_sig,
+				   slock_alloc, slock_free,
+				   slock_lock, slock_unlock, os_hnd);
     if (rv) {
 	ipmi_posix_thread_free_os_handler(os_hnd);
 	os_hnd = NULL;
